@@ -1,5 +1,5 @@
 /*
- * Copyright (C) Pedram Pourang (aka Tsu Jan) 2019 <tsujan2000@gmail.com>
+ * Copyright (C) Pedram Pourang (aka Tsu Jan) 2019-2022 <tsujan2000@gmail.com>
  *
  * FeatherNotes is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -16,7 +16,7 @@
  */
 
 #include "pref.h"
-#include "ui_predDialog.h"
+#include "ui_prefDialog.h"
 
 #include "fn.h"
 
@@ -40,14 +40,16 @@ FNKeySequenceEdit::FNKeySequenceEdit (QWidget *parent) : QKeySequenceEdit (paren
 void FNKeySequenceEdit::keyPressEvent (QKeyEvent *event)
 { // also a workaround for a Qt bug that makes Meta a non-modifier
     clear(); // no multiple shortcuts
+    int k = event->key();
+    if ((event->modifiers() != Qt::NoModifier && event->modifiers() != Qt::KeypadModifier))
+    {
+        if (k == Qt::Key_Super_L || k == Qt::Key_Super_R)
+            return;
+    }
     /* don't create a shortcut without modifier because
        this is a text editor but make exceptions for Fx keys */
-    int k = event->key();
-    if ((k < Qt::Key_F1 || k > Qt::Key_F35)
-        && (event->modifiers() == Qt::NoModifier || event->modifiers() == Qt::KeypadModifier))
-    {
+    else if (k < Qt::Key_F1 || k > Qt::Key_F35)
         return;
-    }
     QKeySequenceEdit::keyPressEvent (event);
 }
 /*************************/
@@ -63,16 +65,23 @@ QWidget* Delegate::createEditor (QWidget *parent,
 /*************************/
 bool Delegate::eventFilter (QObject *object, QEvent *event)
 {
-    QWidget *editor = qobject_cast<QWidget*>(object);
+    FNKeySequenceEdit *editor = qobject_cast<FNKeySequenceEdit*>(object);
     if (editor && event->type() == QEvent::KeyPress)
     {
-        int k = static_cast<QKeyEvent *>(event)->key();
+        QKeyEvent *ke = static_cast<QKeyEvent *>(event);
+        int k = ke->key();
         if (k == Qt::Key_Return || k == Qt::Key_Enter)
         {
             emit QAbstractItemDelegate::commitData (editor);
             emit QAbstractItemDelegate::closeEditor (editor);
             return true;
         }
+        /* treat Tab and Backtab like other keys */
+        if (k == Qt::Key_Tab || k ==  Qt::Key_Backtab)
+        {
+            editor->pressKey (ke);
+            return true;
+        }
     }
     return QStyledItemDelegate::eventFilter (object, event);
 }
@@ -82,6 +91,7 @@ PrefDialog::PrefDialog (QWidget *parent)
 {
     ui->setupUi (this);
     parent_ = parent;
+    ui->promptLabel->setStyleSheet ("QLabel {background-color: #7d0000; color: white; border-radius: 3px; margin: 2px; padding: 5px;}");
     ui->promptLabel->hide();
     promptTimer_ = nullptr;
 
@@ -131,41 +141,13 @@ PrefDialog::PrefDialog (QWidget *parent)
         /* splitter position */
         ui->splitterSizeBox->setChecked (win->isSplitterRem());
         connect (ui->splitterSizeBox, &QCheckBox::stateChanged, win, [win] (int checked) {
-            if (checked == Qt::Checked)
-            {
-                win->remSplitter (true);
-                win->setSplitterSizes (win->getSpltiterState());
-            }
-            else if (checked == Qt::Unchecked)
-                win->remSplitter (false);
+            win->remSplitter (checked == Qt::Checked);
         });
 
         /* window position */
         ui->positionBox->setChecked (win->isPositionRem());
         connect (ui->positionBox, &QCheckBox::stateChanged, win, [this, win] (int checked) {
-            if (checked == Qt::Checked)
-            {
-                win->remPosition (true);
-                win->setPosition (win->geometry().topLeft());
-                if (win->isUnderE())
-                {
-                    ui->ELabel->setEnabled (true);
-                    ui->XLabel->setEnabled (true);
-                    ui->ESpinX->setEnabled (true);
-                    ui->ESpinY->setEnabled (true);
-                }
-            }
-            else if (checked == Qt::Unchecked)
-            {
-                win->remPosition (false);
-                if (win->isUnderE())
-                {
-                    ui->ELabel->setEnabled (false);
-                    ui->XLabel->setEnabled (false);
-                    ui->ESpinX->setEnabled (false);
-                    ui->ESpinY->setEnabled (false);
-                }
-            }
+            win->remPosition (checked == Qt::Checked);
         });
 
         /* use tray */
@@ -219,34 +201,6 @@ PrefDialog::PrefDialog (QWidget *parent)
                 win->showMenubar (true);
         });
 
-        /* Enlightenment */
-        ui->EBox->setChecked (win->isUnderE());
-        ui->ESpinX->setValue (win->EShift().width());
-        ui->ESpinY->setValue (win->EShift().height());
-        if (!win->isUnderE() || !win->isPositionRem())
-        {
-            ui->ELabel->setDisabled (true);
-            ui->ESpinX->setDisabled (true);
-            ui->XLabel->setDisabled (true);
-            ui->ESpinY->setDisabled (true);
-        }
-        connect (ui->ESpinX, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), win, [win] (int value) {
-            win->setEShift (QSize (value, win->EShift().height())); // takes effect in FN::showEvent()
-        });
-        connect (ui->ESpinY, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), win, [win] (int value) {
-            win->setEShift (QSize (win->EShift().width(), value)); // takes effect in FN::showEvent()
-        });
-        connect (ui->EBox, &QCheckBox::stateChanged, win, [this, win] (int checked) {
-            bool isChecked (checked == Qt::Checked);
-            win->setUnderE (isChecked);
-            ui->ESpinX->setEnabled (isChecked);
-            ui->ESpinY->setEnabled (isChecked);
-            ui->ELabel->setEnabled (isChecked);
-            ui->XLabel->setEnabled (isChecked);
-            if (isChecked)
-                win->setEShift (QSize (ui->ESpinX->value(), ui->ESpinY->value()));
-        });
-
         /************
          *** Text ***
          ************/
@@ -279,6 +233,9 @@ PrefDialog::PrefDialog (QWidget *parent)
             showPrompt();
         });
 
+        ui->dateEdit->setPlaceholderText (locale().dateTimeFormat());
+        ui->dateEdit->setText (win->getDateFormat());
+
         /* auto-saving */
         ui->autoSaveSpinBox->setRange (1, 60); // not needed
         if (win->getAutoSave() > -1)
@@ -307,10 +264,35 @@ PrefDialog::PrefDialog (QWidget *parent)
             }
         });
 
-        /* scroll jump workaround */
-        ui->workaroundBox->setChecked (win->isScrollJumpWorkaroundEnabled());
-        connect (ui->workaroundBox, &QCheckBox::stateChanged, win, [win] (int checked) {
-            win->enableScrollJumpWorkaround (checked == Qt::Checked);
+        /* starting with the last opened file */
+        ui->lastFileBox->setChecked (win->openLastFile());
+        connect (ui->lastFileBox, &QCheckBox::stateChanged, win, [win] (int checked) {
+            win->setOpenLastFile (checked == Qt::Checked);
+        });
+
+        /* the number of recently opend files */
+        ui->recentSpin->setValue (win->getRecentFilesNumber());
+        connect (ui->recentSpin, static_cast<void (QSpinBox::*)(int)>(&QSpinBox::valueChanged), win, [win] (int value) {
+            win->setRecentFilesNumber (value); // will take effect at closeEvent()
+        });
+        /* whether recent files should be opened separately */
+        ui->recentBox->setChecked (win->getOpenReccentSeparately()); // up-to-date because of FN::getRecentFilesNumber()
+        connect (ui->recentBox, &QCheckBox::stateChanged, win, [win] (int checked) {
+            win->setOpenReccentSeparately (checked == Qt::Checked);
+        });
+
+        /* whether expanded states of nodes should be remembered */
+        ui->expandBox->setChecked (win->getRememberExpanded());
+        connect (ui->expandBox, &QCheckBox::stateChanged, win, [win] (int checked) {
+            win->setRememberExpanded (checked == Qt::Checked);
+            if (checked == Qt::Checked)
+                win->setCollapsedStates();
+        });
+
+        /* saving on exiting */
+        ui->exitSaveBox->setChecked (win->getSaveOnExit());
+        connect (ui->exitSaveBox, &QCheckBox::stateChanged, win, [win] (int checked) {
+            win->setSaveOnExit (checked == Qt::Checked);
         });
 
         /* spell checking */
@@ -396,14 +378,9 @@ PrefDialog::PrefDialog (QWidget *parent)
         });
     }
 
-    const QString EToolTip = ui->ELabel->toolTip();
-    ui->ESpinX->setToolTip (EToolTip);
-    ui->ESpinY->setToolTip (EToolTip);
-    ui->XLabel->setToolTip (EToolTip);
-
     /* set tooltip as "whatsthis" */
     const auto widgets = findChildren<QWidget*>();
-    for (QWidget *w : widgets)
+    for (const auto &w : widgets)
     {
         QString tip = w->toolTip();
         if (!tip.isEmpty())
@@ -456,10 +433,18 @@ void PrefDialog::onClosing()
         }
         win->updateCustomizableShortcuts();
 
+        QString format = ui->dateEdit->text();
+        /* if "\n" is typed in the line-edit, interpret
+           it as a newline because we're on Linux */
+        if (!format.isEmpty())
+            format.replace ("\\n", "\n");
+        win->setDateFormat (format);
+
         win->setPrefSize (size());
 
         win->writeConfig();
-        win->writeGeometryConfig();
+        win->writeGeometryConfig (false); // doesn't save the last node info
+        win->rememberLastOpenedFile (true); // saves everything about recent files
     }
 }
 /*************************/
@@ -468,12 +453,11 @@ void PrefDialog::prefSize (int value)
     FN *win = static_cast<FN *>(parent_);
     if (win == nullptr) return;
 
-    if (qobject_cast<QCheckBox *>(QObject::sender()))
+    if (qobject_cast<QCheckBox*>(QObject::sender()))
     {
         if (value == Qt::Checked)
         {
             win->remSize (true);
-            win->setWinSize (win->size());
             ui->winSpinX->setEnabled (false);
             ui->winSpinY->setEnabled (false);
             ui->winLabel->setEnabled (false);
@@ -489,9 +473,9 @@ void PrefDialog::prefSize (int value)
             ui->winXLabel->setEnabled (true);
         }
     }
-    else if (ui->winSpinX == qobject_cast<QSpinBox *>(QObject::sender()))
+    else if (ui->winSpinX == qobject_cast<QSpinBox*>(QObject::sender()))
         win->setStartSize (QSize (ui->winSpinX->value(), win->getStartSize().height()));
-    else if (ui->winSpinY == qobject_cast<QSpinBox *>(QObject::sender()))
+    else if (ui->winSpinY == qobject_cast<QSpinBox*>(QObject::sender()))
         win->setStartSize (QSize (win->getStartSize().width(), ui->winSpinY->value()));
 }
 /*************************/
@@ -604,11 +588,9 @@ void PrefDialog::showPrompt (const QString& str, bool temporary)
     FN *win = static_cast<FN *>(parent_);
     if (win == nullptr) return;
 
-    static const QString style ("QLabel {background-color: #7d0000; color: white; border-radius: 3px; margin: 2px; padding: 5px;}");
     if (!str.isEmpty())
     { // show the provided message
         ui->promptLabel->setText ("<b>" + str + "</b>");
-        ui->promptLabel->setStyleSheet (style);
         if (temporary) // show it temporarily
         {
             if (promptTimer_ == nullptr)
@@ -620,7 +602,6 @@ void PrefDialog::showPrompt (const QString& str, bool temporary)
                         && ui->tabWidget->currentIndex() == 3) // Shortcuts page
                     { // show the previous message if it exists
                         ui->promptLabel->setText (prevtMsg_);
-                        ui->promptLabel->setStyleSheet (style);
                     }
                     else showPrompt();
                 });
@@ -635,20 +616,17 @@ void PrefDialog::showPrompt (const QString& str, bool temporary)
              || autoReplace_ != win->hasAutoReplace())
     {
         ui->promptLabel->setText ("<b>" + tr ("Application restart is needed for changes to take effect.") + "</b>");
-        ui->promptLabel->setStyleSheet (style);
     }
     else
     {
         if (prevtMsg_.isEmpty()) // clear prompt
         {
             ui->promptLabel->clear();
-            ui->promptLabel->setStyleSheet ("QLabel {margin: 2px; padding: 5px;}");
+            ui->promptLabel->hide();
+            return;
         }
         else // show the previous message
-        {
             ui->promptLabel->setText (prevtMsg_);
-            ui->promptLabel->setStyleSheet (style);
-        }
     }
     ui->promptLabel->show();
 }
